from datetime import datetime
from pprint import pprint

from django.conf import settings
from django.http import HttpResponse
import adal
from django.shortcuts import redirect
from django.urls import reverse
from django.contrib.auth import authenticate, get_user_model, login as django_login
from django_adal.models import AuthToken
from django_adal.msgraph import get_authorization_url, AUTHORITY_URL, MSGRAPH, graph_get_me, graph_get_my_groups


# Create your views here.
def login(request):
    authorization_url = get_authorization_url(request)
    resp = HttpResponse(status=307)
    resp['location'] = authorization_url
    return resp


def token_exchange(request):
    code = request.GET['code']
    state = request.GET['state']
    if state != request.session.get('state'):
        raise ValueError("State does not match")
    auth_context = adal.AuthenticationContext(AUTHORITY_URL)
    token_response = auth_context.acquire_token_with_authorization_code(code,
                                                                        request.build_absolute_uri(reverse('token')),
                                                                        MSGRAPH,
                                                                        settings.CLIENT_ID,
                                                                        settings.CLIENT_SECRET)
    # It is recommended to save this to a database when using a production app.
    request.session['token'] = token_response

    graph_data = graph_get_me(request)
    request.session['graph_me'] = graph_data

    user_model = get_user_model()
    try:
        # try to retrieve user from database and update it
        usr = user_model.objects.get(username=token_response['userId'])
    except user_model.DoesNotExist:
        # user does not exist in database already, create it
        usr = user_model()
        usr.username = token_response['userId']

    usr.first_name = graph_data['givenName']
    usr.last_name = graph_data['surname']
    usr.email = graph_data['mail']

    # update existing or new user with LDAP data
    usr.set_password(token_response['accessToken'])
    usr.last_login = datetime.now()
    usr.save()

    try:
        # try to retrieve user from database and update it
        token = AuthToken.objects.get(username=token_response['userId'])
    except AuthToken.DoesNotExist:
        # user does not exist in database already, create it
        token = AuthToken()
        token.username = token_response['userId']

    token.token = request.session['token']['accessToken']
    token.save()

    # management of groups
    graph_data = graph_get_my_groups(request)
    request.session['graph_groups'] = graph_data

    # authenticate against Django
    user = authenticate(request=request, username=token_response['userId'],
                        password=token_response['accessToken'])

    if user is not None:
        django_login(request, user)
    else:
        redirect('login')

    return redirect(settings.HOME_REDIRECTION)



from setuptools import setup

setup(
    name='django-adal',
    version='0.1',
    packages=['django_adal'],
    url='https://github.com/GoVanguard/django_adal',
    license='GPL V3',
    author='Shane Scott',
    author_email='sscott@gvit.com',
    description='Very simple authentication module for python2 / django / Azur AD - Office365 using MS ADAL Lib. Forked from Lucterios2/django_auth_adal.',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    install_requires=["Django>=1.11.22", "adal>=1.2.0"],
)



import uuid
from pprint import pprint

import requests
from django.conf import settings
from django.urls import reverse

MSGRAPH = "https://graph.microsoft.com"  # Add the resource you want the access token for
AUTHORITY_HOST_URL = "https://login.microsoftonline.com"

# These settings are for the Microsoft Graph API Call
API_VERSION = 'v1.0'

AUTHORITY_URL = AUTHORITY_HOST_URL + '/' + settings.TENANT
TEMPLATE_AUTHZ_URL = ('https://login.microsoftonline.com/{}/oauth2/authorize?' +
                      'response_type=code&client_id={}&redirect_uri={}&' +
                      'state={}&resource={}')


def get_authorization_url(request):
    auth_state = str(uuid.uuid4())
    request.session['state'] = auth_state

    return TEMPLATE_AUTHZ_URL.format(
        settings.TENANT,
        settings.CLIENT_ID,
        request.build_absolute_uri(reverse('token')),
        auth_state,
        MSGRAPH)


def graph_get(request, end_point_uri):
    endpoint = MSGRAPH + '/' + API_VERSION + end_point_uri
    token = request.session['token']
    print("------------------------------------------------------------------------------------")
    print("GRAPH_GET %s" % end_point_uri)
    pprint(token)
    print("------------------------------------------------------------------------------------")
    http_headers = {'Authorization': 'Bearer ' + token['accessToken'],
                    'User-Agent': 'django_adal',
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'client-request-id': str(uuid.uuid4())}
    return requests.get(endpoint, headers=http_headers, stream=False).json()


def graph_get_me(request):
    return graph_get(request, '/me/')


def graph_get_my_groups(request):
    return graph_get(request, '/me/memberOf')



# -*- coding: utf-8 -*-
from datetime import datetime
import logging
from pprint import pprint

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.exceptions import ObjectDoesNotExist

from django_adal.models import AuthToken

logger = logging.getLogger(__name__)


class MSADALAuth(object):
    def __init__(self):
        self.use_groups = True
        if hasattr(settings, 'ADAL_USE_GROUPS') and isinstance(settings.ADAL_USE_GROUPS, bool):
            self.use_groups = settings.ADAL_USE_GROUPS

    def get_user(self, user_id):
        user_model = get_user_model()
        try:
            return user_model.objects.get(pk=user_id)
        except user_model.DoesNotExist:
            return None

    def authenticate(self, request, username=None, password=None):
        user_model = get_user_model()
        try:
            # try to retrieve user from database and update it
            usr = user_model.objects.get(username=username)
            if AuthToken.objects.get(username=username, token=password):

                # try to get groups information
                if 'graph_groups' in request.session:
                    # if we want to use ADAL group membership:
                    if self.use_groups:
                        logger.info("AUDIT LOGIN FOR: %s AT %s USING ADAL GROUPS" % (username, datetime.now()))
                        # check for groups membership
                        # first cleanup
                        alter_superuser_membership = False
                        if hasattr(settings, 'ADAL_SUPERUSER_GROUPS') and isinstance(settings.ADAL_SUPERUSER_GROUPS,
                                                                                     list) \
                                and len(settings.ADAL_SUPERUSER_GROUPS) > 0:
                            usr.is_superuser = False
                            alter_superuser_membership = True

                        alter_staff_membership = False
                        if hasattr(settings, 'ADAL_STAFF_GROUPS') and isinstance(settings.ADAL_STAFF_GROUPS, list) \
                                and len(settings.ADAL_STAFF_GROUPS) > 0:
                            usr.is_staff = False
                            alter_staff_membership = True

                        usr.save()
                        logger.info(
                            "AUDIT LOGIN FOR: %s AT %s CLEANING OLD GROUP MEMBERSHIP" % (username, datetime.now()))
                        if hasattr(settings, 'ADAL_IGNORED_LOCAL_GROUPS'):
                            grps = Group.objects.exclude(name__in=settings.ADAL_IGNORED_LOCAL_GROUPS)
                        else:
                            grps = Group.objects.all()
                        for grp in grps:
                            grp.user_set.remove(usr)
                            grp.save()

                        # then re-fill
                        if 'value' in request.session['graph_groups']:
                            for group in request.session['graph_groups']['value']:
                                if 'id' in group:
                                    logger.info("AUDIT LOGIN FOR: %s AT %s DETECTED IN GROUP %s" %
                                                (username, datetime.now(), group['displayName']))
                                    # special super user group
                                    if alter_superuser_membership:
                                        if group['id'] in settings.ADAL_SUPERUSER_GROUPS:
                                            usr.is_superuser = True
                                            logger.info("AUDIT LOGIN FOR: %s AT %s GRANTING ADMIN RIGHTS" %
                                                        (username, datetime.now()))
                                        else:
                                            logger.info("AUDIT LOGIN FOR: %s AT %s DENY ADMIN RIGHTS" %
                                                        (username, datetime.now()))
                                    # special staff group
                                    if alter_staff_membership:
                                        if group['id'] in settings.ADAL_STAFF_GROUPS:
                                            usr.is_staff = True
                                            logger.info("AUDIT LOGIN FOR: %s AT %s GRANTING STAFF RIGHTS" %
                                                        (username, datetime.now()))
                                        else:
                                            logger.info("AUDIT LOGIN FOR: %s AT %s DENY STAFF RIGHTS" %
                                                        (username, datetime.now()))
                                    # other groups membership
                                    for grp in settings.ADAL_GROUPS_MAP.keys():
                                        if group['id'] == settings.ADAL_GROUPS_MAP[grp]:
                                            try:
                                                logger.info(grp)
                                                usr.groups.add(Group.objects.get(name=grp))
                                                logger.info("AUDIT LOGIN FOR: %s AT %s ADDING GROUP %s MEMBERSHIP" %
                                                            (username, datetime.now(), grp))
                                            except ObjectDoesNotExist:
                                                pass
                        usr.save()

                    # if set, apply min group membership
                    logger.info("AUDIT LOGIN FOR: %s AT %s BEFORE MIN GROUP MEMBERSHIP" %
                                (username, datetime.now()))
                    if hasattr(settings, 'ADAL_MIN_GROUPS'):
                        for grp in settings.ADAL_MIN_GROUPS:
                            logger.info("AUDIT LOGIN FOR: %s AT %s MIN GROUP MEMBERSHIP: %s" %
                                        (username, datetime.now(), grp))
                            try:
                                usr.groups.add(Group.objects.get(name=grp))
                                logger.info("AUDIT LOGIN FOR: %s AT %s ADDING GROUP %s MIN MEMBERSHIP" %
                                            (username, datetime.now(), grp))
                            except ObjectDoesNotExist:
                                pass

                return usr
        except user_model.DoesNotExist:
            # user does not exist in database already, create it
            return None
        except AuthToken.DoesNotExist:
            return None
